/*
	MIT License http://www.opensource.org/licenses/mit-license.php
	Author Tobias Koppers @sokra
*/
var loaderUtils = require("loader-utils");

module.exports = function() {};
module.exports.pitch = function(remainingRequest) {
	this.cacheable && this.cacheable();
	var query = loaderUtils.parseQuery(this.query);
	if(query.name) {
		var options = {
			context: query.context || this.options.context,
			regExp: query.regExp
		};
		var chunkName = loaderUtils.interpolateName(this, query.name, options);
		var chunkNameParam = ", " + JSON.stringify(chunkName);		
	} else {
		var chunkNameParam = '';
	}
	var result;
	if(query.lazy) {
		result = [
			"var fn = function(onSuccessCb, onErrorCb) {\n",
			"	var result = require.ensure([], function(require) {\n",
			"		onSuccessCb(require(", loaderUtils.stringifyRequest(this, "!!" + remainingRequest), "));\n",
			"	}" + chunkNameParam + ")\n;",
			"	if (result.catch && onErrorCb) {\n",
			"		result.catch(onErrorCb);\n",
			"	}\n",
			"};\n",
			"\n",
			"fn.__asyncModule = true;\n",
			"fn.__lazyModule = true;\n",
			"module.exports = fn;"];
	} else {
		result = [
			"var onSuccessCbs = [],\n",
			"var onErrorCbs = [],\n",
			"	data;\n",
			"	error;\n",
			"var fn = function(onSuccessCb, onErrorCb) {\n",
			"	if(onSuccessCbs) onSuccessCbs.push(onSuccessCb);\n",
			"	else onSuccessCb(data);\n",
			"	if(onErrorCbs) onErrorCbs.push(onErrorCb);\n",
			"	else if (onErrorCb) onErrorCb(error);\n",
			"}\n",
			"var result = require.ensure([], function(require) {\n",
			"	data = require(", loaderUtils.stringifyRequest(this, "!!" + remainingRequest), ");\n",
			"	var callbacks = onSuccessCbs;\n",
			"	onSuccessCbs = null;\n",
			"	for(var i = 0, l = callbacks.length; i < l; i++) {\n",
			"		callbacks[i](data);\n",
			"	}\n",
			"}" + chunkNameParam + ");\n",
			"if (result.catch) {\n",
			"	result.catch(function(err) {\n",
			"		error = err;\n",
			"		var callbacks = onErrorCbs;\n",
			"		onErrorCbs = null;\n",
			"		for(var i = 0, l = callbacks.length; i < l; i++) {\n",
			"			callbacks[i] && callbacks[i](error);\n",
			"		}\n",
			"	});\n",
			"};\n",
			"fn.__asyncModule = true;\n",
			"module.exports = fn;"];
        ];
	}
	return result.join("");
}

/*
Output format:

	var onSuccessCbs = [],
	var onErroCbs = [],
		data;
		error;
	module.exports = function(onSuccessCb, onErrorCb) {
		if(onSuccessCbs) onSuccessCbs.push(onSuccessCb);
		else onSuccessCb(data);
		if(onErrorCbs) onErrorCbs.push(onErrorCb);
		else if (onErrorCb) onErrorCb(error);
	}
	var result = require.ensure([], function(require) {
		data = require("xxx");
		var callbacks = onSuccessCbs;
		onSuccessCbs = null;
		for(var i = 0, l = callbacks.length; i < l; i++) {
			callbacks[i](data);
		}
	});
	if (result.catch) {
		result.catch(function(err) {
			error = err;
			var callbacks = onErrorCbs;
			onErrorCbs = null;
			for(var i = 0, l = callbacks.length; i < l; i++) {
				calbacks[i] && callbacks[i](error);
			}
		});
	});

*/
